@@ -685,8 +685,18 @@ QList<Target*> DatabaseManager::getTargets(QString filter)
 {
 	QList<Target*> result;
 
-	QSqlQuery query("select id, name, type, groupName from targets");
-	//QSqlQuery query("select id, name, type, picture, pictureBack  from targets");
+	QSqlQuery query;
+	bool ret;
+	if (filter != NULL && filter.length() > 0) {
+		ret = query.prepare("select id, name, type, groupName from targets where groupName = :groupName");
+		query.bindValue(":groupName", filter);
+	}
+	else {
+		ret = query.prepare("select id, name, type, groupName from targets");
+	}
+
+	ret = query.exec ();
+
 	while (query.next()) {
 		Target * item = new Target();
 		item->id = query.value(0).toInt();
@@ -696,36 +706,24 @@ QList<Target*> DatabaseManager::getTargets(QString filter)
 		item->pictureSmall = NULL;
 		item->picture = NULL;
 		item->pictureBack = NULL;
-/*
-		if (query.value(3).isNull()) {
-			item->picture = NULL;
-		} else {
-			item->picture = new QImage ();
-			QByteArray im_s;
-			decrypt(query.value(3).toByteArray(), im_s);
-			if (!item->picture->loadFromData (im_s)) {
-				qDebug () << "Error loading image " << item->id;
-			} else
-				qDebug () << "loaded image " << item->id;
-		}
-		if (query.value(4).isNull()) {
-			item->pictureBack = NULL;
-		} else {
-			item->pictureBack = new QImage ();
-			QByteArray im_s;
-			decrypt(query.value(4).toByteArray(), im_s);
-			if (!item->pictureBack->loadFromData (im_s)) {
-				qDebug () << "Error loading bimage " << item->id;
-			} else
-				qDebug () << "loaded bimage " << item->id;
-		}
-*/
 		result.append(item);
 	}
 
 	return result;
 }
 
+QList<QString> DatabaseManager::getGroups()
+{
+	QList<QString> result;
+
+	QSqlQuery query("select distinct groupName from targets order by groupName");
+	while (query.next()) {
+		result.append(query.value(0).toString());
+	}
+
+	return result;
+}
+
 bool DatabaseManager::deleteTarget(Target * target)
 {
 	bool ret = false;
