@@ -15,6 +15,7 @@
  * TODO multilanguage
  * TODO groups implementation
  * TODO pc version?
+ * TODO fix export and import to add field type
  */
 
 #include "eWallet.h"
@@ -38,6 +39,7 @@ DatabaseManager * dbManager;
 eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 {
 	centralWidget = NULL;
+	isShowGroups = 0;
 
 	this->setAttribute(Qt::WA_Maemo5AutoOrientation, true);
 	connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged(int)));
@@ -47,6 +49,8 @@ eWallet::eWallet(QWidget *parent) : QMainWindow(parent)
 	ascSort = true;
 	currentFilter = "";
 
+	grabGesture(Qt::SwipeGesture);
+
 	//this->setAttribute(Qt::WA_Maemo5AutoOrientation, true);
 
 	loadTypesAndFields ();
@@ -123,23 +127,85 @@ void eWallet::showList () {
 	qDebug () << "setFocus()";
 	centralWidget->setFocus ();
 
-	showTargets (true);
+	showInfo (true);
+}
+
+void eWallet::changeMode () {
+	isShowGroups = (isShowGroups) % 2 + 1;
+	_cancelSearch ();
+	showInfo(true);
+}
+
+void eWallet::swipeTriggered(QSwipeGesture *gesture) {
+	if (gesture->state() == Qt::GestureFinished) {
+		if (gesture->horizontalDirection() == QSwipeGesture::Left
+			|| gesture->verticalDirection() == QSwipeGesture::Up)
+			isShowGroups = (isShowGroups + 1) % 2 + 1;
+		else
+			isShowGroups = (isShowGroups + 1) % 2 + 1;
+		showInfo(false);
+	}
+}
+
+bool eWallet::gestureEvent(QGestureEvent *event) {
+    if (QGesture *swipe = event->gesture(Qt::SwipeGesture))
+        swipeTriggered(static_cast<QSwipeGesture *>(swipe));
+//    else if (QGesture *pan = event->gesture(Qt::PanGesture))
+//        panTriggered(static_cast<QPanGesture *>(pan));
+//    if (QGesture *pinch = event->gesture(Qt::PinchGesture))
+//        pinchTriggered(static_cast<QPinchGesture *>(pinch));
+    return true;
+}
+
+
+bool eWallet::event(QEvent *event)
+{
+    if (event->type() == QEvent::Gesture) {
+    	qDebug () << "GESTURE";
+
+    	if (!isAskPassword) {
+    		return gestureEvent(static_cast<QGestureEvent*>(event));
+    	}
+    }
+    return QWidget::event(event);
 }
 
 void eWallet::selectItem(QListWidgetItem * item) {
-	if (fnListWidget->currentItem() != item) {
-		fnListWidget->setCurrentItem(item);
+	if (isShowGroups == 1) {
+		if (fnListWidget->currentItem() != item) {
+			fnListWidget->setCurrentItem(item);
+		}
+
+		QString target = grouplist.at(fnListWidget->currentRow());
+		isShowGroups = 2;
+		showTargets(reset, target);
+	}
+	else {
+		if (fnListWidget->currentItem() != item) {
+			fnListWidget->setCurrentItem(item);
+		}
+		Target * target = list.at(fnListWidget->currentRow());
+		viewTarget(target);
 	}
-	Target * target = list.at(fnListWidget->currentRow());
-	viewTarget(target);
 }
 
 void eWallet::selectDItem(QListWidgetItem * item) {
-	if (fnListWidget->currentItem() != item) {
-		fnListWidget->setCurrentItem(item);
+	if (isShowGroups == 1) {
+		if (fnListWidget->currentItem() != item) {
+			fnListWidget->setCurrentItem(item);
+		}
+
+		QString target = grouplist.at(fnListWidget->currentRow());
+		isShowGroups = 2;
+		showTargets(reset, target);
+	}
+	else {
+		if (fnListWidget->currentItem() != item) {
+			fnListWidget->setCurrentItem(item);
+		}
+		Target * target = list.at(fnListWidget->currentRow());
+		editTarget(target);
 	}
-	Target * target = list.at(fnListWidget->currentRow());
-	editTarget(target);
 }
 
 void eWallet::listenInactivity(QString s) {
@@ -150,6 +216,7 @@ void eWallet::listenInactivity(QString s) {
 		// TODO sure????
 		fnListWidget->clear();
 		list.clear ();
+		grouplist.clear ();
 
 		if (!detailWindow.isNull()) {
 			detailWindow->close();
@@ -180,24 +247,44 @@ void eWallet::textChanged (const QString & text) {
 }
 
 void eWallet::filterBy (const QString & s) {
-	for (int i = 0; i < list.size (); i++) {
-		Target * target = list.at(i);
+	if (isShowGroups == 1) {
+		for (int i = 0; i < grouplist.size (); i++) {
+			QString target = grouplist.at(i);
 
-		if (s.length() == 0) {
-			fnListWidget->item(i)->setHidden(false);
-		}
-		else {
-			if (target->name.contains(s, Qt::CaseInsensitive)) {
+			if (s.length() == 0) {
 				fnListWidget->item(i)->setHidden(false);
 			}
 			else {
-				fnListWidget->item(i)->setHidden(true);
+				if (target.contains(s, Qt::CaseInsensitive)) {
+					fnListWidget->item(i)->setHidden(false);
+				}
+				else {
+					fnListWidget->item(i)->setHidden(true);
+				}
+			}
+		}
+	}
+	else {
+		for (int i = 0; i < list.size (); i++) {
+			Target * target = list.at(i);
+
+			if (s.length() == 0) {
+				fnListWidget->item(i)->setHidden(false);
+			}
+			else {
+				if (target->name.contains(s, Qt::CaseInsensitive)) {
+					fnListWidget->item(i)->setHidden(false);
+				}
+				else {
+					fnListWidget->item(i)->setHidden(true);
+				}
 			}
 		}
 	}
 }
 
 void eWallet::_cancelSearch () {
+	currentFilter = "";
 	searchField->setText("");
 	searchField->hide();
 	cancelSearch->hide();
@@ -403,18 +490,73 @@ QPixmap eWallet::getImage (Target * target) {
 	}
 	return image;
 }
-void eWallet::showTargets(bool reset)
+
+void eWallet::showGroups(bool reset)
+{
+	qDebug () << "show Groups " << reset;
+
+	// Clean previous items from the layout
+	fnListWidget->clear();
+
+	if (reset) {
+		grouplist.clear ();
+		grouplist = ::dbManager->getGroups ();
+	}
+
+//	if (ascSort)
+//		qSort (grouplist.begin(),grouplist.end(),sortTargetsASC);
+//	else
+//		qSort (grouplist.begin(),grouplist.end(),sortTargetsDES);
+
+	//fnListWidget->setIconSize (QSize (SMALL_WIDTH, SMALL_HEIGH));
+	//fnListWidget->setContentsMargins(10, 1, 0, 1);
+
+	for (int i = 0; i < grouplist.size (); i++) {
+		QString target = grouplist.at(i);
+
+		QListWidgetItem* listitem = new QListWidgetItem(target, fnListWidget);
+		//listitem->set(target->getSummary());
+		fnListWidget->addItem(listitem);
+	}
+
+	//filterBy(currentFilter);
+
+	fnListWidget->setCurrentRow(0);
+
+	//update();
+}
+
+void eWallet::showInfo(bool reset) {
+	if (isShowGroups == 0) {
+		if (reset) {
+			grouplist.clear ();
+			grouplist = ::dbManager->getGroups ();
+		}
+
+		if (grouplist.size() > 1)
+			isShowGroups = 1;
+		else
+			isShowGroups = 2;
+	}
+
+	if (isShowGroups == 1) {
+		showGroups(reset);
+	}
+	else {
+		showTargets(reset, "");
+	}
+}
+
+void eWallet::showTargets(bool reset, const QString & filter)
 {
 	qDebug () << "show Targets " << reset;
 
 	// Clean previous items from the layout
 	fnListWidget->clear();
 
-	//QPixmap available(":/resources/icon_available.png");
-	//QPixmap notavailable(":/resources/icon_notavailable.png");
 	if (reset) {
 		list.clear ();
-		list = ::dbManager->getTargets ("");
+		list = ::dbManager->getTargets (filter);
 	}
 
 	if (ascSort)
@@ -432,12 +574,6 @@ void eWallet::showTargets(bool reset)
 		//listitem->set(target->getSummary());
 		fnListWidget->addItem(listitem);
 		
-//		QObject::connect(listitem, SIGNAL(viewTarget(Target*)),
-//                      this, SLOT(viewTarget(Target*)));
-//		QObject::connect(listitem, SIGNAL(editTarget(Target*)),
-//                      this, SLOT(editTarget(Target*)));
-//		QObject::connect(listitem, SIGNAL(deleteTarget(Target*)),
-//                      this, SLOT(deleteTarget(Target*)));
 	}
 
 	filterBy(currentFilter);
@@ -466,7 +602,7 @@ void eWallet::createMenu()
 		menuBar()->addAction("Invert Sort", this, SLOT(sort()));
 		menuBar()->addAction("Import", this, SLOT(import()));
 		menuBar()->addAction("Export", this, SLOT(exportData()));
-		menuBar()->addAction("About", this, SLOT(about()));
+		menuBar()->addAction("Change Mode", this, SLOT(changeMode()));
 		menuBar()->addAction("Exit", QApplication::instance(), SLOT(quit()));
 	}
 }
@@ -474,7 +610,7 @@ void eWallet::createMenu()
 void eWallet::sort()
 {
 	ascSort = !ascSort;
-	showTargets(false);
+	showInfo(false);
 }
 
 bool eWallet::__insertTarget(Target* target)
@@ -535,14 +671,17 @@ bool eWallet::_deleteTarget(Target* target)
 
 
 	int i = list.indexOf(target);
-	//qDebug () << "DELETE ON " << i;
+	qDebug () << "DELETE ON " << i;
 	if (::dbManager->deleteTarget(target)) {
+		qDebug () << "trying to remove " << i;
 		list.removeAt(i);
 
+		qDebug () << "trying to takeItem " << i;
 		fnListWidget->takeItem(i); //oveItem (row);
 //		target->freeFieldList();
 //		target->freeResources();
 //		delete target;
+		qDebug () << "delete " << i;
 		delete target;
 		return true;
 	}
@@ -587,7 +726,7 @@ bool eWallet::_updateTargetPictureBack(Target* target)
 
 void eWallet::deleteTarget(Target * target) {
 	if (_deleteTarget (target)) {
-		delete target;
+		//delete target;
 	}
 }
 
@@ -769,7 +908,7 @@ void eWallet::import() {
 		    // Close the file
 		    file.close();
 
-		    showTargets (true);
+		    showTargets (true, "");
 		  }
 	}
 	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
@@ -795,15 +934,20 @@ void eWallet::newTarget() {
 void eWallet::contextMenuEvent (QContextMenuEvent * event) {
 	//qDebug () << "Should be show " << this->childAt(event->globalPos());
 
-	if (fnListWidget->currentRow() >= 0) {
-	Target * target = list.at(fnListWidget->currentRow());
+	if (isShowGroups == 1) {
 
-		QMenu menu(this);
-		menu.addAction("View", this, SLOT(_viewcTarget()));
-		menu.addAction("Edit", this, SLOT(_editcTarget()));
-		menu.addAction("New like this", this, SLOT(_newLikeTarget()));
-		menu.addAction("Delete", this, SLOT(_deletecTarget()));
-		menu.exec(event->globalPos());
+	}
+	else {
+		if (fnListWidget->currentRow() >= 0) {
+		Target * target = list.at(fnListWidget->currentRow());
+
+			QMenu menu(this);
+			menu.addAction("View", this, SLOT(_viewcTarget()));
+			menu.addAction("Edit", this, SLOT(_editcTarget()));
+			menu.addAction("New like this", this, SLOT(_newLikeTarget()));
+			menu.addAction("Delete", this, SLOT(_deletecTarget()));
+			menu.exec(event->globalPos());
+		}
 	}
 }
 
