@@ -44,12 +44,19 @@ public slots:
 	void selectDItem(QListWidgetItem *);
 
 	void deleteDB ();
+	void changeMode ();
 
 private:
+	bool event (QEvent *event);
+	void swipeTriggered (QSwipeGesture *gesture);
+	bool gestureEvent (QGestureEvent *event);
+
 	void showList ();
 	void keyPressEvent(QKeyEvent*);
 	void createMenu();
-	void showTargets(bool reset);
+	void showGroups(bool reset);
+	void showInfo(bool reset);
+	void showTargets(bool reset, const QString & filter);
 	void loadTypesAndFields ();
 	void filterBy (const QString & s);
 	QPixmap getImage (Target * f);
@@ -80,6 +87,7 @@ private:
 	int times;
 	QListWidget* fnListWidget;
 	QList<Target *> list;
+	QList<QString> grouplist;
 
 	QHBoxLayout * inputWidget;
 	QPushButton * cancelSearch;
@@ -94,7 +102,7 @@ private:
 	bool opendb;
 	bool initdb;
 	bool ascSort;
-
+	int isShowGroups;
 	QString currentFilter;
 
 };
